/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* 
* @defgroup gpiote_example_main main.c
* @{
* @ingroup nrf_gpiote_example
* @brief GPIOTE Example Application main file.
*
* This file contains the source code for a sample application using GPIOTE. 
*/

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "nrf_gpiote.h"
#include "nrf_gpio.h"
#include "boards.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"
#include "nrf_drv_gpiote.h"
#include "app_error.h"


//--------------------
#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "app_button.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp.h"
#include "bsp_btn_ble.h"
#include "ble_radio_notification.h"
//nrf_ppi_channel_t ppi_channel;
#define IS_SRVC_CHANGED_CHARACT_PRESENT 0                                           /**< Include the service_changed characteristic. If not enabled, the server's database cannot be changed for the lifetime of the device. */

#define DEVICE_NAME                     "Nordic_UART"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_ADV_INTERVAL                800//800 is 500msec //8000//4800//4800*0.625 = 3 seconds//64//1600//1sec////64//3200//64                                          /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS      180                                         /**< The advertising timeout (in units of seconds). */

#define APP_TIMER_PRESCALER             0                                           /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE         4                                           /**< Size of timer operation queues. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER)  /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER) /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define START_STRING                    "Start...\n"                                /**< The string that will be sent over the UART when the application starts. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */

static ble_nus_t                        m_nus;                                      /**< Structure to identify the Nordic UART Service. */
static uint16_t                         m_conn_handle = BLE_CONN_HANDLE_INVALID;    /**< Handle of the current connection. */

static ble_uuid_t                       m_adv_uuids[] = {{BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}};  /**< Universally unique service identifier. */

static nrf_drv_timer_t timer = NRF_DRV_TIMER_INSTANCE(2);
const nrf_drv_timer_t STREAM_PAC = NRF_DRV_TIMER_INSTANCE(2);
static volatile uint8_t on_turn = 0;
static volatile ret_code_t err_code3;
static volatile nrf_ppi_channel_t ppi_channel;
static volatile nrf_ppi_channel_t ppi_channelN;
static volatile nrf_ppi_channel_t ppi_channelP;
#define GPIO_OUTPUT_PIN_NUMBER 29//12 
#define GPIO_OUTPUT_PIN_NUMBER_N	13//28
#define GPIO_OUTPUT_PIN_NUMBER_P 12
uint32_t compare_evt_addr;
uint32_t gpiote_task_addr;
uint32_t compare_evt_addrN;
uint32_t gpiote_task_addrN;	
uint32_t compare_evt_addrP;
uint32_t gpiote_task_addrP;		
static volatile uint32_t new_cycle =0;
void  timer_stream_event_handler (nrf_timer_event_t event_type);
#define BSTR_PWM_ON	40//30//40
#define BSTR_PWM_OFF	30//20//30
#define PWMN_ON	130
#define PWMP_ON	130
/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse 
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of 
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    
    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_nus    Nordic UART Service structure.
 * @param[in] p_data   Data to be send to UART module.
 * @param[in] length   Length of the data.
 */
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_t * p_nus, uint8_t * p_data, uint16_t length)
{
    for (uint32_t i = 0; i < length; i++)
    {
        while(app_uart_put(p_data[i]) != NRF_SUCCESS);
    }
    while(app_uart_put('\n') != NRF_SUCCESS);
}
/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t       err_code;
    ble_nus_init_t nus_init;
    
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;
    
    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;
    
    if(p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;
    
    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;
    
    err_code = ble_conn_params_init(&cp_init);
  //  APP_ERROR_CHECK(err_code);//hack
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            break;
        case BLE_ADV_EVT_IDLE:
            break;
        default:
            break;
    }
}


/**@brief Function for the Application's S110 SoftDevice event handler.
 *
 * @param[in] p_ble_evt S110 SoftDevice event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t                         err_code;
    
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            break;
            
        case BLE_GAP_EVT_DISCONNECTED:
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        default:
            // No implementation needed.
            break;
    }
}
void reinit2(void)
{

    uint32_t time_ticks;
//    uint32_t err_code;

    ret_code_t err_code2;
		uint32_t err_code;
	
    err_code2 = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code2);

    err_code2 = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code2);	
	
	
  //  err_code = nrf_drv_timer_init(&STREAM_PAC, NULL, timer_stream_event_handler);
   // APP_ERROR_CHECK(err_code);

  //  time_ticks = 160;//16;//nrf_drv_timer_ms_to_ticks(&STREAM_PAC, time_ms);

 //   nrf_drv_timer_extended_compare(&STREAM_PAC, NRF_TIMER_CC_CHANNEL0, time_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);

	
#ifdef NRF51
    //Workaround for P5AN-73.
    *(uint32_t *)0x40008C0C = 1;
#endif


    
    nrf_drv_gpiote_out_config_t config = GPIOTE_CONFIG_OUT_TASK_TOGGLE(false);

    err_code3 = nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER, &config);
    APP_ERROR_CHECK(err_code3);

    err_code3 = nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER_N, &config);
    APP_ERROR_CHECK(err_code3);

    err_code3 = nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER_P, &config);
    APP_ERROR_CHECK(err_code3);


//    nrf_drv_timer_extended_compare(&timer, (nrf_timer_cc_channel_t)0, 200*1000UL, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, false);


    err_code3 = nrf_drv_ppi_channel_alloc(&ppi_channel);
    APP_ERROR_CHECK(err_code3);
		
    err_code3 = nrf_drv_ppi_channel_alloc(&ppi_channelN);
    APP_ERROR_CHECK(err_code3);		

    err_code3 = nrf_drv_ppi_channel_alloc(&ppi_channelP);
    APP_ERROR_CHECK(err_code3);		

    compare_evt_addr = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addr = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER);
		
    compare_evt_addrN = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addrN = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER_N);		
		
    compare_evt_addrP = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addrP = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER_P);			

		err_code3 = nrf_drv_ppi_channel_assign(ppi_channel, compare_evt_addr, gpiote_task_addr);
    APP_ERROR_CHECK(err_code3);
		
    err_code3 = nrf_drv_ppi_channel_assign(ppi_channelN, compare_evt_addrN, gpiote_task_addrN);
    APP_ERROR_CHECK(err_code3);		

    err_code3 = nrf_drv_ppi_channel_assign(ppi_channelP, compare_evt_addrP, gpiote_task_addrP);
    APP_ERROR_CHECK(err_code3);		

    err_code3 = nrf_drv_ppi_channel_enable(ppi_channel);
    APP_ERROR_CHECK(err_code3);
		
//    err_code3 = nrf_drv_ppi_channel_enable(ppi_channelN);
//    APP_ERROR_CHECK(err_code3);		

    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER);
		
    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER_N);		
    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER_P);		

	
  //  nrf_drv_timer_enable(&STREAM_PAC);	
}
void reinit(void)
{

    uint32_t time_ticks;
//    uint32_t err_code;

    ret_code_t err_code2;
		uint32_t err_code;
	
   nrf_drv_ppi_init();
    

   nrf_drv_gpiote_init();
    
	
	
  //  err_code = nrf_drv_timer_init(&STREAM_PAC, NULL, timer_stream_event_handler);
   // APP_ERROR_CHECK(err_code);

  //  time_ticks = 160;//16;//nrf_drv_timer_ms_to_ticks(&STREAM_PAC, time_ms);

 //   nrf_drv_timer_extended_compare(&STREAM_PAC, NRF_TIMER_CC_CHANNEL0, time_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);

	
#ifdef NRF51
    //Workaround for P5AN-73.
    *(uint32_t *)0x40008C0C = 1;
#endif


    
    nrf_drv_gpiote_out_config_t config = GPIOTE_CONFIG_OUT_TASK_TOGGLE(false);

    nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER, &config);
    

    nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER_N, &config);
    

    nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER_P, &config);
    


//    nrf_drv_timer_extended_compare(&timer, (nrf_timer_cc_channel_t)0, 200*1000UL, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, false);


    nrf_drv_ppi_channel_alloc(&ppi_channel);
    
		
    nrf_drv_ppi_channel_alloc(&ppi_channelN);
   		

    nrf_drv_ppi_channel_alloc(&ppi_channelP);
   

    compare_evt_addr = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addr = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER);
		
    compare_evt_addrN = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addrN = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER_N);		
		
    compare_evt_addrP = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addrP = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER_P);			

		nrf_drv_ppi_channel_assign(ppi_channel, compare_evt_addr, gpiote_task_addr);
    
		
   nrf_drv_ppi_channel_assign(ppi_channelN, compare_evt_addrN, gpiote_task_addrN);
   		

    nrf_drv_ppi_channel_assign(ppi_channelP, compare_evt_addrP, gpiote_task_addrP);
   

    nrf_drv_ppi_channel_enable(ppi_channel);
  
		
//    err_code3 = nrf_drv_ppi_channel_enable(ppi_channelN);
//    APP_ERROR_CHECK(err_code3);		

    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER);
		
    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER_N);		
    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER_P);		

	
  //  nrf_drv_timer_enable(&STREAM_PAC);	
}

/**@brief Function for dispatching a S110 SoftDevice event to all modules with a S110 SoftDevice 
 *        event handler.
 *
 * @details This function is called from the S110 SoftDevice event interrupt handler after a S110 
 *          SoftDevice event has been received.
 *
 * @param[in] p_ble_evt  S110 SoftDevice event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    ble_conn_params_on_ble_evt(p_ble_evt);
    ble_nus_on_ble_evt(&m_nus, p_ble_evt);
    on_ble_evt(p_ble_evt);
    ble_advertising_on_ble_evt(p_ble_evt);
    
}
void clearPWMpins(void)
{
			nrf_gpio_cfg_output(GPIO_OUTPUT_PIN_NUMBER);
			nrf_gpio_cfg_output(GPIO_OUTPUT_PIN_NUMBER_N);
			nrf_gpio_cfg_output(GPIO_OUTPUT_PIN_NUMBER_P);	
  		nrf_gpio_pin_clear(GPIO_OUTPUT_PIN_NUMBER);
			nrf_gpio_pin_clear(GPIO_OUTPUT_PIN_NUMBER_N);
			nrf_gpio_pin_clear(GPIO_OUTPUT_PIN_NUMBER_P);	
}
/**
*	\brief 	Radio callback if the radio is active.
*	\param[in] radio_active: is 1 in the radio mode, 0 otherwise 
*	\param[out] None.
*
*/
static volatile uint32_t debugcnt =0;
static void ble_on_radio_active_evt(bool radio_active)
{
	ret_code_t err_code3;
	if(radio_active)
	{
			debugcnt++;

		//nrf_drv_ppi_uninit();
   nrf_drv_gpiote_uninit();	
//nrf_gpio_cfg_output(28);
 		
		
			clearPWMpins();
		//reinit2();
	}
	else
	{
		

		on_turn=0;//10;
	reinit2();
	
	}
               
}

/**@brief Function for the S110 SoftDevice initialization.
 *
 * @details This function initializes the S110 SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;
    
    // Initialize SoftDevice.
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, NULL);

    // Enable BLE stack.
    ble_enable_params_t ble_enable_params;
    memset(&ble_enable_params, 0, sizeof(ble_enable_params));
#if (defined(S130) || defined(S132))
    ble_enable_params.gatts_enable_params.attr_tab_size   = BLE_GATTS_ATTR_TAB_SIZE_DEFAULT;
#endif
    ble_enable_params.gatts_enable_params.service_changed = IS_SRVC_CHANGED_CHARACT_PRESENT;
    err_code = sd_ble_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);
    
    // Subscribe for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
	
    err_code = ble_radio_notification_init(NRF_APP_PRIORITY_LOW,
                                           //NRF_RADIO_NOTIFICATION_DISTANCE_3620US,
																					// NRF_RADIO_NOTIFICATION_DISTANCE_800US,//from Vital
																					//NRF_RADIO_NOTIFICATION_DISTANCE_4560US,
																					NRF_RADIO_NOTIFICATION_DISTANCE_5500US,
                                           ble_on_radio_active_evt);
               
    APP_ERROR_CHECK(err_code);		
	
	
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;
    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break;

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break;

        case BSP_EVENT_WHITELIST_OFF:
            err_code = ble_advertising_restart_without_whitelist();
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break;

        default:
            break;
    }
}


/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to 
 *          a string. The string will be be sent over BLE when the last character received was a 
 *          'new line' i.e '\n' (hex 0x0D) or if the string has reached a length of 
 *          @ref NUS_MAX_DATA_LENGTH.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;

            if ((data_array[index - 1] == '\n') || (index >= (BLE_NUS_MAX_DATA_LEN)))
            {
                err_code = ble_nus_string_send(&m_nus, data_array, index);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
                
                index = 0;
            }
            break;

        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
 */
/**@snippet [UART Initialization] */
static void uart_init(void)
{
    uint32_t                     err_code;
    const app_uart_comm_params_t comm_params =
    {
        RX_PIN_NUMBER,
        TX_PIN_NUMBER,
        RTS_PIN_NUMBER,
        CTS_PIN_NUMBER,
        APP_UART_FLOW_CONTROL_ENABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud38400
    };

    APP_UART_FIFO_INIT( &comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);
    APP_ERROR_CHECK(err_code);
}
/**@snippet [UART Initialization] */


/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    ble_advdata_t scanrsp;

    // Build advertising data struct to pass into @ref ble_advertising_init.
    memset(&advdata, 0, sizeof(advdata));
    advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance = false;
    advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

    memset(&scanrsp, 0, sizeof(scanrsp));
    scanrsp.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    scanrsp.uuids_complete.p_uuids  = m_adv_uuids;

    ble_adv_modes_config_t options = {0};
    options.ble_adv_fast_enabled  = BLE_ADV_FAST_ENABLED;
    options.ble_adv_fast_interval = APP_ADV_INTERVAL;
    options.ble_adv_fast_timeout  = APP_ADV_TIMEOUT_IN_SECONDS;

    err_code = ble_advertising_init(&advdata, &scanrsp, &options, on_adv_evt, NULL);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
}


/**@brief Function for placing the application in low power state while waiting for events.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}
void timer_startHFclock(void)
{
    // Start 16 MHz crystal oscillator .
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART    = 1;

    // Wait for the external oscillator to start up.
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0)
    {
        // Do nothing.
    }	
}
#define BLE_PART 
#define GPIOTE_PART 

void  timer_stream_event_handler (nrf_timer_event_t event_type)
{
    uint8_t   i;
    uint32_t   err_code;

    switch(event_type)
    {
        case NRF_TIMER_EVENT_COMPARE0:
				if(on_turn==0)
				{
					on_turn++;
					NRF_TIMER2->CC[0] = BSTR_PWM_ON*16;
				}
				else if(on_turn==1)
				{
					on_turn++;
					NRF_TIMER2->CC[0] = BSTR_PWM_OFF*16;
				}
				else if(on_turn==2)
				{
					on_turn++;
					NRF_TIMER2->CC[0] = BSTR_PWM_ON*16;
				}
				else if(on_turn==3)
				{
					//it takes 22usec to execute the disabling and enabling of channels.
					//Since the minimum period without glitch is 30usec, the interval
					//between end of booster pwm and start of pwmN is set to be 50usec
					on_turn++;
					
				//disable booster pwm and enable PWMN
						nrf_drv_ppi_channel_disable(ppi_channel);
					
						nrf_drv_ppi_channel_enable(ppi_channelN);
						NRF_TIMER2->CC[0] = 50*16;//50*16;
				}
				else if(on_turn==4)
				{
					on_turn++;
					NRF_TIMER2->CC[0] = PWMN_ON*16;
				}
				else if(on_turn==5)
				{
					//It takes 22usec to execute the disabling and enabling of channels.
					//Since the minimum period without glitch is 30usec, the interval
					//between end of pwmN and start of pwmP is set to be 50usec						
					on_turn++;
				
				//disable booster pwm and enable PWMN
						nrf_drv_ppi_channel_disable(ppi_channelN);
					
						nrf_drv_ppi_channel_enable(ppi_channelP);
						NRF_TIMER2->CC[0] = 50*16;//50*16;
				}
				else if(on_turn==6)
				{
					on_turn++;
					NRF_TIMER2->CC[0] = PWMP_ON*16;
				}
				else if(on_turn==7)
				{
					on_turn=0;
					//disable PWMN and enable Booster PWM
					
					
						nrf_drv_ppi_channel_enable(ppi_channel);
						//APP_ERROR_CHECK(err_code3);		
					
						nrf_drv_ppi_channel_disable(ppi_channelP);
						//APP_ERROR_CHECK(err_code3);		
						NRF_TIMER2->CC[0] = 4000*16;
				}

        break;
        default:
        //Do nothing.
        break;
    }
}

/**@brief Application main function.
 */
int main(void)
{
	
      uint32_t err_code;
	timer_startHFclock();
#ifdef BLE_PART
	
    // Initialize.

		uart_init();
		//uint32_t opt_id = BLE_COMMON_OPT_RADIO_CPU_MUTEX;
		//ble_opt_t cpu_blocking_enabled;
		//cpu_blocking_enabled.common_opt.radio_cpu_mutex.enable = 0;
//https://devzone.nordicsemi.com/question/18751/how-to-unblock-the-cpu-during-connection-intervals-with-s110-v710/
    ble_stack_init();
		//err_code = sd_ble_opt_set(opt_id, &cpu_blocking_enabled);	
	
    gap_params_init();
    services_init();
    advertising_init();
    conn_params_init();
    

    err_code = ble_advertising_start(BLE_ADV_MODE_FAST);
    APP_ERROR_CHECK(err_code);
#endif	
#ifdef GPIOTE_PART	
    uint32_t time_ms = 1;//Time(in miliseconds) between consecutive compare events.
    uint32_t time_ticks;
		nrf_gpio_cfg_output(28);
    ret_code_t err_code2;

    err_code2 = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code2);

    err_code2 = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code2);	
	
	nrf_drv_timer_config_t configint;
	configint.interrupt_priority=1;//3;//1;
    err_code = nrf_drv_timer_init(&STREAM_PAC, &configint, timer_stream_event_handler);
    APP_ERROR_CHECK(err_code);

    time_ticks = 160;//10 microseconds;//nrf_drv_timer_ms_to_ticks(&STREAM_PAC, time_ms);

    nrf_drv_timer_extended_compare(&STREAM_PAC, NRF_TIMER_CC_CHANNEL0, time_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);

	
#ifdef NRF51
    //Workaround for P5AN-73.
    *(uint32_t *)0x40008C0C = 1;
#endif

    // Setup PPI channel with event from TIMER compare and task GPIOTE pin toggle.
    uint32_t compare_evt_addr;
    uint32_t gpiote_task_addr;
	  uint32_t compare_evt_addrN;
    uint32_t gpiote_task_addrN;	
	  uint32_t compare_evt_addrP;
    uint32_t gpiote_task_addrP;	
    
    nrf_drv_gpiote_out_config_t config = GPIOTE_CONFIG_OUT_TASK_TOGGLE(false);

    err_code3 = nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER, &config);
    APP_ERROR_CHECK(err_code3);

    err_code3 = nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER_N, &config);
    APP_ERROR_CHECK(err_code3);

    err_code3 = nrf_drv_gpiote_out_init(GPIO_OUTPUT_PIN_NUMBER_P, &config);
    APP_ERROR_CHECK(err_code3);



    err_code3 = nrf_drv_ppi_channel_alloc(&ppi_channel);
    APP_ERROR_CHECK(err_code3);
		
    err_code3 = nrf_drv_ppi_channel_alloc(&ppi_channelN);
    APP_ERROR_CHECK(err_code3);		

    err_code3 = nrf_drv_ppi_channel_alloc(&ppi_channelP);
    APP_ERROR_CHECK(err_code3);		

    compare_evt_addr = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addr = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER);
		
    compare_evt_addrN = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addrN = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER_N);		
		
    compare_evt_addrP = nrf_drv_timer_event_address_get(&timer, NRF_TIMER_EVENT_COMPARE0);
    gpiote_task_addrP = nrf_drv_gpiote_out_task_addr_get(GPIO_OUTPUT_PIN_NUMBER_P);			

		err_code3 = nrf_drv_ppi_channel_assign(ppi_channel, compare_evt_addr, gpiote_task_addr);
    APP_ERROR_CHECK(err_code3);
		
    err_code3 = nrf_drv_ppi_channel_assign(ppi_channelN, compare_evt_addrN, gpiote_task_addrN);
    APP_ERROR_CHECK(err_code3);		

    err_code3 = nrf_drv_ppi_channel_assign(ppi_channelP, compare_evt_addrP, gpiote_task_addrP);
    APP_ERROR_CHECK(err_code3);		

    err_code3 = nrf_drv_ppi_channel_enable(ppi_channel);
    APP_ERROR_CHECK(err_code3);
		


    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER);
		
    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER_N);		
    nrf_drv_gpiote_out_task_enable(GPIO_OUTPUT_PIN_NUMBER_P);		
	
	
    nrf_drv_timer_enable(&STREAM_PAC);

		#endif	
    while (1)
    {
            //power_manage();
    }
}

/** @} */
